/*
*  RAL -- Rubrica Addressbook Library
*  file: abook.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <libintl.h>
#include <glib.h>
#include <glib-object.h>
#include <glib/gstdio.h>

#include "rlib.h"
#include "manager.h"
#include "plugin.h"
#include "abook.h"
#include "card.h"
#include "personal.h"
#include "company.h"
#include "group_set.h"
#include "utils.h"
#include "error.h"


/*  property enumeration
 */
enum {
  ABOOK_NAME = 1,
  ABOOK_PATH,
  DELETED_CARDS,
  SELECTED_ID,
};


struct _RAbookPrivate {
  gchar* name;              /* the addressbook (the file) name     */
  gchar* path;              /* the addressbook (the file) path     */
  gint   deleted;           /* cards marked deleted in addressbook */
  gulong selected_id;       /* selected card id                    */

  RGroupSet* set;

  GList* cards;
  GList* iter;

  RPlugin* r_plugin;          // RPlugin 
  gpointer plugin;            // ptr to real plugin (a gobject)
  RPluginManager* manager;

  gboolean dispose_has_run;
};


/*    signals enumeration 
 */
enum {
  OPEN_FAIL,            /* file not opened     */
  SAVE_FAIL,            /* file not saved      */

  ADDRESSBOOK_CHANGED,  /* addressbook changed */
  ADDRESSBOOK_READ,     /* file read           */ 
  ADDRESSBOOK_SAVED,    /* file saved          */
  ADDRESSBOOK_CLOSED,       
  
  CARD_READ,            /* card read                                        */ 
  CARD_ADDED,           /* card added to addressbook                        */
  CARD_REMOVED,         /* card removed from addressbook and moved to trash */
  CARD_DESTROYED,       /* card destroyed                                   */
  CARD_RECOVERED,       /* card recoverd from trash                         */
  CARD_REPLACED,        /* card replaced in addressbook                     */
  
  CARD_CUTTED,
  CARD_PASTED,

  LAST_SIGNAL 
};

static guint r_abook_signals[LAST_SIGNAL] = {0};

 
static void r_abook_class_init   (RAbookClass* klass);
static void r_abook_init         (RAbook* obj);

static void r_abook_dispose      (RAbook* obj);
static void r_abook_finalize     (RAbook* obj);

static void r_abook_set_property (GObject* obj, guint property_id,
				  const GValue* value, GParamSpec* spec);
static void r_abook_get_property (GObject* obj, guint property_id,
				  GValue* value, GParamSpec* spec);


gboolean   r_abook_recovery_last_card_from_trash (RAbook* abook);




GType
r_abook_get_type()
{
  static GType r_abook_type = 0;
  
  if (!r_abook_type)
    {
      static const GTypeInfo r_abook_info =
	{
	  sizeof(RAbookClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) r_abook_class_init,
	  NULL,
	  NULL,
	  sizeof(RAbook),
	  0,
	  (GInstanceInitFunc) r_abook_init
	};

      r_abook_type = g_type_register_static (G_TYPE_OBJECT, "RAbook",
					     &r_abook_info, 0);
    }
  
  return r_abook_type;
}


static void
r_abook_class_init(RAbookClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose         = (GObjectFinalizeFunc) r_abook_dispose;
  class->finalize        = (GObjectFinalizeFunc) r_abook_finalize;
  class->set_property    = r_abook_set_property;
  class->get_property    = r_abook_get_property;

  klass->abook_open_file = NULL;
  klass->abook_save_file = NULL;
  klass->abook_overwrite = NULL;


  /* class signals 
   */   
  /**
   * RAbook::open-fail:
   * @abook: the #RAbook object that receives the signal
   * @err: an #RError
   *
   * the "open-fail" signal is emitted if the given file can't be read
   */
  r_abook_signals[OPEN_FAIL] =
    g_signal_new("open_fail", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, open_fail),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_INT);            /* params type: error code */

  /**
   * RAbook::save-fail:
   * @abook: the #RAbook object that receives the signal
   * @err: an #RError
   *
   * the "save-fail" signal is emitted if the given file can't be read 
   */
  r_abook_signals[SAVE_FAIL] =
    g_signal_new("save_fail", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, save_fail),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_INT);            /* params type: error code */

  /**
   * RAbook::card-read:
   * @abook: the #RAbook object that receives the signal
   * @data: a pointer to the readed card 
   * 
   * The "card-read" signal is emitted after the addressbook has been readed
   */
  r_abook_signals[CARD_READ] = 
    g_signal_new("card_read", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_read),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);
   
  /**
   * RAbook::card-added:
   * @abook: the #RAbook object that receives the signal
   * @data: a pointer to the readed card 
   *
   * The "card-added" signal is emitted after a card is added to addressbook.
   * A pointer to the card is passed to the callback function
   */
  r_abook_signals[CARD_ADDED] =
    g_signal_new("card_added", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_added),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);
  
  /**
   * RAbook::card-removed:
   * @abook: the #RAbook object that receives the signal
   *
   * The "card-removed" signal is emitted when a card is 
   * (deleted) removed from addressbook. The removed cards are 
   * marked as deleted, to really delete (remove) the card 
   * you must destroy it. You can recover (deleted) removed cards.
   * Destroyed cards are unrecoverable
   */
  r_abook_signals[CARD_REMOVED] =
    g_signal_new("card_removed", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_removed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);

  /**
   * RAbook::card-destroyed:
   * @abook: the #RAbook object that receives the signal
   *
   * The "card-destroyed" signal is emitted when a card is 
   * deleted from addressbook. Destroyed cards are unrecoverable.
   */
  r_abook_signals[CARD_DESTROYED] =
    g_signal_new("card_destroyed", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_destroyed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);

  /**
   * RAbook::card-recoverd:
   * @abook: the #RAbook object that receives the signal
   *
   * The "card-recovered" signal is emitted when a previously deleted 
   * marked card is unmarked. 
   */
  r_abook_signals[CARD_RECOVERED] =
    g_signal_new("card_recovered", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_recovered),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);

  /**
   * RAbook::card-replaced:
   * @abook: the #RAbook object that receives the signal
   * @data: a pointer to the readed card 
   *
   * The "card-replaced" signal is emitted a card replacement.
   * A pointer to the new card is passed to the callback function
   */
  r_abook_signals[CARD_REPLACED] =
    g_signal_new("card_replaced", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_replaced),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);

  /**
   * RAbook::card-cutted
   *
   *
   *
   *
   */
  r_abook_signals[CARD_CUTTED] =
    g_signal_new("card_cutted", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, card_cutted),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__POINTER,
		 G_TYPE_NONE,
		 1,
		 G_TYPE_POINTER);

  
  /**
   * RAbook::addressbook-read:
   * @abook: the #RAbook object that receives the signal
   *
   * The "addressbook-read" signal is emitted when the 
   * addressbook is read
   */
  r_abook_signals[ADDRESSBOOK_READ] =
    g_signal_new("addressbook_read", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, addressbook_read),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);

  /**
   * RAbook::addressbook_saved:
   * @abook: the #RAbook object that receives the signal
   *
   * The "addressbook_saved" signal
   */
  r_abook_signals[ADDRESSBOOK_SAVED] =
    g_signal_new("addressbook_saved", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, addressbook_saved),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);

  /**
   * RAbook::addressbook_changed:
   * @abook: the #RAbook object that receives the signal
   *
   * The "addressbook_changed" signal
   */
  r_abook_signals[ADDRESSBOOK_CHANGED] =
    g_signal_new("addressbook_changed", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, addressbook_changed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);


  /**
   * RAbook::addressbook_closed:
   * @abook: the #RAbook object that receives the signal
   *
   * The "addressbook_closed" signal
   */
  r_abook_signals[ADDRESSBOOK_CLOSED] =
    g_signal_new("addressbook_closed", 
		 R_ABOOK_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RAbookClass, addressbook_closed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__VOID,
		 G_TYPE_NONE,
		 0,
		 G_TYPE_NONE);


  /**
   * ABook:addressbook-name
   *
   * address book's name
   */
  pspec = g_param_spec_string("addressbook-name",
			      "addressbook's name",
			      "the name (filename) of the addressbook",
			      _("no name"),
			      G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
  g_object_class_install_property(class, ABOOK_NAME, pspec);     

  /**
   * ABook:addressbook-path
   *
   * address book's path
   */
  pspec = g_param_spec_string("addressbook-path",
			      "addressbook's path",
			      "the path of the addressbook",
			      NULL,
			      G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
  g_object_class_install_property(class, ABOOK_PATH, pspec);


  /**
   * ABook:deleted-cards
   *
   * the number of cards marked as deleted in the addressbook
   */
  pspec = g_param_spec_int("deleted-cards",
			   "deleted cards",
			   "number of cards marked as deleted "
			   "in the addressbook",
			   G_MININT,
			   G_MAXINT,
			   0,
			   G_PARAM_READABLE);
  g_object_class_install_property(class, DELETED_CARDS, pspec);


  /**
   * ABook:selected-id
   *
   * load/save the id of the selected card
   */
  pspec = g_param_spec_long("selected-id",
			    "selected card's id",
			    "load/save the id of the selected card",
			    G_MINLONG,
			    G_MAXLONG,
			    0,
			    G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
  g_object_class_install_property(class, SELECTED_ID, pspec);
}


static void 
r_abook_set_property (GObject* obj, guint property_id,
		      const GValue* value, GParamSpec* spec)
{
  RAbook *self = (RAbook*) obj;
  
  switch(property_id)
    {
    case ABOOK_NAME:
      g_free(self->private->name);
      self->private->name = g_value_dup_string(value);
      break;
      
    case ABOOK_PATH:
      g_free(self->private->path);
      self->private->path = g_value_dup_string(value);
      break;
      
    case DELETED_CARDS:
      break;

    case SELECTED_ID: 
      self->private->selected_id = g_value_get_long(value);
      break;
      
    default:
      break;    
    }
}


static void 
r_abook_get_property (GObject* obj, guint property_id,
		      GValue* value, GParamSpec* spec)
{
  RAbook *self = (RAbook*) obj;
  
  switch(property_id)
    {
    case ABOOK_NAME:
      g_value_set_string(value, self->private->name);
      break;
      
    case ABOOK_PATH:
      g_value_set_string(value, self->private->path);
      break;
      
    case DELETED_CARDS:
      g_value_set_int(value, self->private->deleted);
      break;

    case SELECTED_ID: 
      g_value_set_long(value, self->private->selected_id);
      break;

    default:
      break;    
    }
}


static void
r_abook_init(RAbook* self)
{
  self->private = g_new(RAbookPrivate, 1);

  self->private->name        = g_strdup(gettext("no name"));
  self->private->path        = NULL;
  self->private->selected_id = 0L;
  self->private->deleted     = 0;

  self->private->set         = r_lib_get_group_set();
 
  self->private->cards       = NULL;
  self->private->iter        = NULL;

  self->private->r_plugin    = NULL;
  self->private->plugin      = NULL;
  self->private->manager     = r_lib_get_manager();
  
  r_lib_admit (self);
  
  self->private->dispose_has_run = FALSE;
}

 

static void 
r_abook_dispose (RAbook* self)
{
  g_return_if_fail(IS_R_ABOOK(self));
  
  /* free the list's items (PersonalCard or CompanyCard) 
   */
  self->private->iter = self->private->cards;
  for (; self->private->iter; self->private->iter = self->private->iter->next)
    {
      RCard* card = self->private->iter->data;

      if (IS_R_CARD(card))
	r_card_free(R_CARD(card));
    }
  
  g_list_free(self->private->cards);
  self->private->cards = NULL;
  self->private->iter  = NULL;
  
  r_plugin_free(self->private->r_plugin);
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
r_abook_finalize (RAbook* self)
{
  g_return_if_fail(IS_R_ABOOK(self));
  
  r_utils_free_string(self->private->name);
  r_utils_free_string(self->private->path);

  g_free(self->private);
  self->private = NULL;
}



/*   ***************************** Public *****************************
*/


/**
 * r_abook_new
 *
 * create a #RAbook
 *
 * Returns: a new #RAbook
 */
RAbook*
r_abook_new(void)
{
  RAbook* abook;

  if (!r_lib_is_running())
    g_error("\nLibral is not running");

  abook = g_object_new(r_abook_get_type(), NULL);
  
  return abook;
}


/**
 * r_abook_free
 * @abook: a #RAbook
 * 
 * free the #RAbook
 * The "addressbook_closed" signal is emitted 
 */
void
r_abook_free(RAbook* abook)
{
  g_return_if_fail(IS_R_ABOOK(abook));

  g_signal_emit_by_name(abook, "addressbook_closed", NULL, G_TYPE_NONE);

  g_object_unref(abook);   
}



/**
 * r_abook_open_file
 * @abook: a #RAbook
 * @fname: the name of the addressbook
 *
 * open the addressbook with given name
 *
 * Returns: %TRUE if file is opened successfully, %FALSE otherwise
 */
gboolean 
r_abook_open_file (RAbook* abook, gchar* fname)
{
  RAbookClass* class;

  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);

  class = R_ABOOK_GET_CLASS(abook);
  
  if (class->abook_open_file)
    return class->abook_open_file(abook, fname); 
  else
    {
      if (g_str_has_suffix(fname, "rub"))
	{
	  if (r_abook_load_plugin(abook, "rubrica"))
	    return class->abook_open_file(abook, fname);
	}
      
      if (g_str_has_suffix(fname, "vcr") || g_str_has_suffix(fname, "vcard"))
	{
	  if (r_abook_load_plugin(abook, "vcard"))
	    return class->abook_open_file(abook, fname);
	}
      
      if (g_str_has_suffix(fname, "csv"))
	{
	  if (r_abook_load_plugin(abook, "csv"))
	    return class->abook_open_file(abook, fname);
	}	
    }

  return FALSE;
}



/**
 * r_abook_save_file
 * @abook: a #RAbook
 * @fname: the name of file
 * 
 * save the addressbook with given filename
 *
 * Returns: %TRUE if file is saved successfully, %FALSE otherwise
 */
gboolean 
r_abook_save_file (RAbook* abook, gchar* fname, gint compression_rate)
{
  RAbookClass* class;

  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);

  class = R_ABOOK_GET_CLASS(abook);
  
  if (class->abook_save_file)
    {
      gboolean result;
      
      result = class->abook_save_file(abook, fname, compression_rate); 
      if (result)
	{
	  g_object_set(abook, 
		       "addressbook-name", g_path_get_basename(fname),
		       "addressbook-path", g_path_get_dirname(fname), NULL);
	  
	  g_signal_emit_by_name(abook, "addressbook_saved", NULL, G_TYPE_NONE);
	}
      
      return result;
    }

  return FALSE;
}


/**
 * r_abook_overwrite_file
 * @abook: a #RAbook
 * @backup: boolean
 *
 * overwrite an existing addressbook with a new one.
 * If backup is %TRUE the old addressbook is backuped.
 *
 * Returns: a %TRUE if old addressbook was successfully overwited, 
 * %FALSE otherwise
 **/
gboolean
r_abook_overwrite_file(RAbook* abook, gboolean backup, gint compression_rate)
{
  gchar *old, *new;
  RAbookClass* class;

  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);

  class = R_ABOOK_GET_CLASS(abook);
  
  if (backup)
    {
      old = g_strdup_printf("%s%s%s", 
			    abook->private->path, 
			    G_DIR_SEPARATOR_S,
			    abook->private->name);
      new = g_strdup_printf("%s~", old);
      g_rename(old, new);
      g_free(new);
      g_free(old);
    }
  
  /* write the file */
  if (class->abook_overwrite)
    return class->abook_overwrite(abook, compression_rate); 
  
  return FALSE;
}


/**
 * r_abook_plugin_from_file
 * @abook: a #RAbook
 * @filename: gchar*
 *
 * Try to load the plugin using the file extension. If the file has a 
 * known extension the rigth plugin is loaded.
 * 
 * Returns: %TRUE if plugin is loaded, %FALSE otherwise
 **/
gboolean 
r_abook_plugin_from_file (RAbook* abook, gchar* filename)
{
  GList* filters = NULL;

  filters = r_plugin_manager_get_all_filters(abook->private->manager);
  for (; filters; filters = filters->next)
    {
      gchar *name, *pattern;
      RFilter* filter = (RFilter*) filters->data;
      
      g_object_get(filter, "filter-name", &name, NULL);

      r_filter_reset(filter);
      pattern = r_filter_get_pattern(filter);
      for (; pattern; pattern = r_filter_get_next_pattern(filter))
	if (g_str_has_suffix(filename, pattern))
	  {
	    gchar* name;

	    g_object_get(filter, "filter-name", &name, NULL);
	    return r_abook_load_plugin (abook, name);
	  }
    }
  
  return FALSE;  
}


/**
 * r_abook_load_plugin
 * @abook: a #RAbook
 * @plugin_name: const gchar*
 *
 * Load the "plugin_name" plugin.
 * 
 * Returns: %TRUE if plugin is loaded, %FALSE otherwise
 **/
gboolean 
r_abook_load_plugin (RAbook* abook, const gchar* plugin_name)
{
  RPlugin* plugin;
  RAbookClass* class = NULL;

  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);
  g_return_val_if_fail(plugin_name != NULL, FALSE);
  
  class = R_ABOOK_GET_CLASS(abook);
  if (!class)
    {
      g_warning("\nR_ABOOK_GET_CLASS");
      
      return FALSE;
    }

  plugin = r_plugin_manager_get_plugin(abook->private->manager, plugin_name);
  if (plugin)
    {
      abook->private->r_plugin = plugin;
      abook->private->plugin   = r_plugin_get_obj(plugin);

      class->abook_open_file   = r_plugin_get_handle(plugin, "read");
      class->abook_save_file   = r_plugin_get_handle(plugin, "write");
      class->abook_overwrite   = r_plugin_get_handle(plugin, "overwrite");
      
      return TRUE;
    }    

  return FALSE;
}


/**
 * r_abook_get_plugin
 * @abook: a #RAbook
 * 
 * Get the currently active plugin
 *
 * Returns: a %gpointer to the plugin
 **/	 
gpointer 
r_abook_get_plugin (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);

  return abook->private->plugin;
}

/**
 * r_abook_get_plugin
 * @abook: a #RAbook
 * 
 * Get the currently active plugin
 *
 * Returns: a %gpointer to the plugin
 **/	 
gpointer 
r_abook_get_r_plugin (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);

  return abook->private->r_plugin;
}



/**
 * r_abook_copy
 * @abook: a #RAbook
 *
 * copy the addressbook pointed by abook
 *
 * Returns: a new #RAbook
 */
RAbook* 
r_abook_copy (RAbook* abook)
{
  RAbook* new;
  RPlugin* plugin;
  RAbookClass* class = NULL;
  gpointer card;
  
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
  
  new = r_abook_new();
  
  plugin = r_abook_get_plugin(abook);
  g_object_ref(plugin);
  
  class = R_ABOOK_GET_CLASS(new);
  if (class)
    {
      abook->private->plugin = plugin;

      class->abook_open_file = r_plugin_get_handle(plugin, "read");
      class->abook_save_file = r_plugin_get_handle(plugin, "write");
      class->abook_overwrite = r_plugin_get_handle(plugin, "overwrite");
    }

  card = r_abook_get_card(abook);
  for (; card; card = r_abook_get_next_card(abook))
    {
      gpointer new_card; 

      new_card = r_card_copy(R_CARD(card));
      
      if (new_card)
	r_abook_add_card(new, new_card);
    }    
  r_abook_reset_book(abook);
  
  return new;
}


/**
 * r_abook_get_items
 * @abook: a #RAbook
 *
 * get the number of cards into addressbok
 *
 * Returns: an integer 
 */
gint
r_abook_get_items (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), -1);

  return g_list_length(abook->private->cards);
}


/**
 * r_abook_is_empty
 * @abook: a #RAbook
 *
 * is the addressbook empty?
 *
 * Returns: a boolean. %TRUE if addressbook is empty, %FALSE otherwise
 */
gboolean
r_abook_is_empty (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), TRUE);
  
  return (r_abook_get_items(abook) == 0);
}


/**
 * r_abook_add_card
 * @abook: a #RAbook
 * @card: an #RCard object
 *
 * append the given card into the addressbook
 * The "card_added" and "addressbook_changed" signals are emitted
 */
void   
r_abook_add_card (RAbook* abook, RCard* card)
{
  GList* user_groups = NULL;
  gboolean deleted = FALSE;
		      
  g_return_if_fail(IS_R_ABOOK(abook));

  user_groups = r_card_get_groups_owned_by(R_CARD(card), "user");
  for (; user_groups; user_groups = user_groups->next)
    r_group_set_add_group(abook->private->set, R_GROUP(user_groups->data));

  /* check if the card is marked deleted */
  g_object_get(card, "card-deleted", &deleted, NULL);
  if (deleted)
    abook->private->deleted++;
  
  abook->private->cards = g_list_append(abook->private->cards, card);
  abook->private->iter  = abook->private->cards;

  g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
  g_signal_emit_by_name(abook, "card_added", card, G_TYPE_POINTER);
}

/**
 * r_abook_destroy_card
 * @abook: a #RAbook
 * @card: an #RCard object
 *
 * Destroy the given card. The destroyed cards are unrecoverable
 * The "card_destroyed" and "addressbook_changed" signals are emitted
 */ 
void
r_abook_destroy_card(RAbook* abook, RCard* card)
{
  g_return_if_fail(IS_R_ABOOK(abook));  
  g_return_if_fail(IS_R_CARD(card));
  
  abook->private->cards = g_list_remove(abook->private->cards, card);
  r_card_free(card);
  
  g_signal_emit_by_name(abook, "card_destroyed", NULL, G_TYPE_NONE);
  g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);  
}


/**
 * r_abook_delete_card
 * @abook: a #RAbook
 * @card: an #RCard object
 *
 * mark the given card as delete
 * The "card_removed" and "addressbook_changed" signals are emitted
 */
void
r_abook_delete_card (RAbook* abook, RCard* card)
{
  g_return_if_fail(IS_R_ABOOK(abook));  
  g_return_if_fail(IS_R_CARD(card));
  
  g_object_set(card, "card-deleted", TRUE, NULL);
  abook->private->deleted++;  

  g_signal_emit_by_name(abook, "card_removed", card, G_TYPE_POINTER);
  g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
}


/**
 * r_abook_recovery_card
 * @abook: a #RAbook
 * @card: an #RCard object
 *
 * unmark the card previously marked as deleted. 
 * The "card_recovered" and "addressbook_changed" signals are emitted
 *
 * Returns: %TRUE if card is successfully recovered, %FALSE otherwise
 */
gboolean 
r_abook_recovery_card (RAbook* abook, RCard* card)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);  
  g_return_val_if_fail(IS_R_CARD(card), FALSE);  
  

  g_object_set(card, "card-deleted", FALSE, NULL);
  abook->private->deleted--;
  
  g_signal_emit_by_name(abook, "card_recovered", card, G_TYPE_POINTER); 
  g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
  
  return TRUE;
}

/**
 * r_abook_move_card_to_trash
 * @abook: an #ABook
 * @card: a #RCard
 *
 * move the given card to the trash
 *
 * Returns: %TRUE, if card is moved succesfully to trash, %FALSE otherwise

gboolean 
r_abook_move_card_to_trash (RAbook *abook, RCard* card)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);  
  g_return_val_if_fail(IS_R_CARD(card), FALSE);
  
abook->private->cards = g_list_remove(abook->private->cards, card);
  if(r_trash_append_card(abook->private->trash, abook, card))
    {
      g_object_set(card, "card-deleted", TRUE, NULL);
      abook->private->deleted++;
  
  //  abook->private->iter = abook->private->cards;
  
      g_signal_emit_by_name(abook, "card_removed", card, G_TYPE_POINTER);
      g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);  
      
      return TRUE;
    }


  
  return FALSE;
}
 */

/*
RDelType
r_abook_delete_card_with_refs(RAbook* abook, RCard* card)
{
  GList* deleting = NULL;

  g_return_val_if_fail(IS_R_ABOOK(abook), R_CARD_UNDELETED);  
  g_return_val_if_fail(IS_R_CARD(card), R_CARD_UNDELETED);
  
  // TODO

  return R_CARD_UNDELETED;
}


gboolean   
r_abook_recovery_last_card_from_trash (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), FALSE);  

  if (r_trash_recovery_last_card(abook->private->trash))
    {
      abook->private->deleted--;
      
      return TRUE;
    }

  return FALSE;
}



*/


/**
 * r_abook_replace_card
 * @abook: a #RAbook
 * @old: an #RCard object
 * @new: an #RCard object
 * 
 * replace the old card with the new one
 * The "card_replaced" and "addressbook_changed" signals are emitted
 */
void   
r_abook_replace_card (RAbook* abook, RCard* old, RCard* new)
{
  gint pos;
  GList* node = NULL;
  gpointer card = NULL;

  g_return_if_fail(IS_R_ABOOK(abook));
      
  pos  = g_list_index(abook->private->cards, old);
  node = g_list_nth(abook->private->cards, pos);
  if (node)
    {
      abook->private->cards = g_list_remove_link(abook->private->cards, node);
      
      card = node->data;
      r_card_free(R_CARD(card));
      
      g_list_free_1(node);

      abook->private->cards = g_list_insert(abook->private->cards, new, pos);
      
      g_signal_emit_by_name(abook, "addressbook_changed", NULL, G_TYPE_NONE);
      g_signal_emit_by_name(abook, "card_replaced", new, G_TYPE_POINTER);
    }
}



/**
 * r_abook_get_card
 * @abook: a #RAbook
 *
 * Get the first card in addressbook. 
 *
 * Returns: a pointer to the card or NULL if addressbook is empty
 */
gpointer 
r_abook_get_card (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);

  if (abook->private->iter)
    return abook->private->iter->data;

  return NULL;
}



/**
 * r_abook_get_selected_card
 * @abook: a #RAbook
 * 
 * Get the selected card in the addressbook
 *
 * Returns:
 */
gpointer      
r_abook_get_selected_card (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
 
  return r_abook_get_card_by_id(abook, abook->private->selected_id);
}


/**
 * r_abook_unselect_cards
 * @abook: a #RAbook
 * 
 * unselect card. Clear previpus selections
 */
void 
r_abook_unselect_cards (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);

  abook->private->selected_id = 0L;
}


/**
 * r_abook_get_card_by_id
 * @abook: a #RAbook
 * @id: the card id
 * 
 * get the card with "id" id
 *
 * Returns: a pointer to the card or NULL if addressbook doesn't own that card
 */
gpointer      
r_abook_get_card_by_id (RAbook* abook, glong id)
{
  gpointer card = NULL;

  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
  g_return_val_if_fail(id > 0L, NULL);
   
  r_abook_reset_book(abook);
  for (card = r_abook_get_card(abook); card; 
       card = r_abook_get_next_card(abook))
    {
      glong card_id;
      
      g_object_get(R_CARD(card), "card-id", &card_id, NULL);
      if (id == card_id)
	{
	  abook->private->selected_id = id;
	  
	  return card;      
	}
    }  

  return NULL;
}

/**
 * r_abook_get_group_set(RAbook* abook)
 * @abook: a #RAbook
 * 
 * get the default groups set
 * 
 * returns: a #gpointer (caller must cast to #RGroupSet*) or NULL
 */
gpointer 
r_abook_get_groups_set(RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
  
  return (gpointer) abook->private->set;
}


/**
 * r_abook_find_cards_in_group
 * @abook: a #RAbook
 * @group_name: the name of the group
 *
 * Find all cards that belongs to the given group. 
 * Found cards's ID has inserted in a GList. Caller must free returned GList
 * 
 * returns: a GList* or %NULL if no cards belong to group
 */
GList*
r_abook_find_cards_in_group (RAbook* abook, gchar* group_name)
{
  gpointer data;
  GList* found = NULL;

  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
  g_return_val_if_fail(group_name != NULL, NULL);

  r_abook_reset_book(abook);
  data = r_abook_get_card(abook);
  for (; data; data = r_abook_get_next_card(abook))
    {
      glong id;

      if (r_card_belong_to_group(R_CARD(data), group_name) ||
	  (g_ascii_strcasecmp(group_name, "all groups") == 0))
	{
	  g_object_get(R_CARD(data), "card-id", &id, NULL);
	  
	  found = g_list_append(found, GINT_TO_POINTER(id));
	}
    }
  
  return found;
}



/**
 * r_abook_reset_book
 * @abook: a #RAbook
 * 
 * reset the private addressbook iterator. The iterator 
 * will point the head of cards's list.
 */
void     
r_abook_reset_book (RAbook* abook)
{
  g_return_if_fail(IS_R_ABOOK(abook));
 
  abook->private->iter = abook->private->cards;
}


/**
 * r_abook_get_next_card
 * @abook: a #RAbook
 *
 * get the next card in addressbook
 * 
 * Returns: a pointer to the card or NULL if the cards's end list is reached.
 */
gpointer
r_abook_get_next_card (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
 
  abook->private->iter = g_list_next(abook->private->iter);
  if (abook->private->iter)
    return abook->private->iter->data;
  else
    abook->private->iter = g_list_last(abook->private->cards);  

  return NULL;
}


/**
 * r_abook_get_prev_card
 * @abook: a #RAbook
 *
 * get the previous card in addressbook
 * 
 * Returns: a pointer to the card or NULL if the cards's head list is reached.
 */
gpointer 
r_abook_get_prev_card (RAbook* abook)
{
  g_return_val_if_fail(IS_R_ABOOK(abook), NULL);
 
  abook->private->iter = g_list_previous(abook->private->iter);
  if (abook->private->iter)
    return abook->private->iter->data;
  else 
    abook->private->iter = abook->private->cards;

  return NULL;
}


/**
 * r_abook_foreach_card
 * @abook: a #RAbook
 * @func: the #RFunc that will be called for each card in addressbook
 * @data: user's data to pass to func
 *
 * Call the function func() for each card in the #RAbook. 
 */
void 
r_abook_foreach_card (RAbook* abook, RFunc func, gpointer data)
{
  gpointer card;
     
  r_abook_reset_book(abook);
  card = r_abook_get_card(abook);
  for (; card; card = r_abook_get_next_card(abook))
    func(card, data);    
}
