/*
*  RAL -- Rubrica Addressbook Library
*  file: contact.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdlib.h>
#include <glib.h>
#include <glib-object.h>

#include "contact.h"
#include "date.h"
#include "utils.h"


enum {
  CONTACT_FIRST = 1,
  CONTACT_MIDDLE,
  CONTACT_LAST,
  CONTACT_NICK,
  CONTACT_PROFESSION,
  CONTACT_PREFIX,
  CONTACT_TITLE,
  CONTACT_PHOTO
};


struct _RContactPrivate {
  gchar*   first;          /* first name  */
  gchar*   middle;         /* middle name */
  gchar*   last;           /* last name   */
  gchar*   nick;           /* nick name   */

  gchar*   prof;           /* profession  */
  gchar*   prefix;         /* prefix  (egr. sir. jr. I, II, ...)     */ 
  gchar*   title;          /* title   (sig. sig.ra, sig.na dott, ... */

  RDate* birthday;         /* */

  gchar*   photo;          /* contact's photo (uri) */

  gboolean dispose_has_run;
};



static void r_contact_class_init (RContactClass* klass);
static void r_contact_init       (RContact* obj);

static void r_contact_dispose    (RContact* obj);
static void r_contact_finalize   (RContact* obj);

static void r_contact_set_property (GObject* obj, guint property_id, 
				    const GValue* value, GParamSpec* spec);
static void r_contact_get_property (GObject* obj, guint property_id,
				    GValue* value, GParamSpec* spec);


GType
r_contact_get_type()
{
  static GType contact_type = 0;
  
  if (!contact_type)
    {
      static const GTypeInfo contact_info =
	{
	  sizeof(RContactClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) r_contact_class_init,
	  NULL,
	  NULL,
	  sizeof(RContact),
	  0,
	  (GInstanceInitFunc) r_contact_init
	};

      contact_type = g_type_register_static (G_TYPE_OBJECT, "RContact",
					     &contact_info, 0);
    }
  
  return contact_type;
}


static void
r_contact_class_init(RContactClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) r_contact_dispose;
  class->finalize = (GObjectFinalizeFunc) r_contact_finalize;

  class->set_property = r_contact_set_property;
  class->get_property = r_contact_get_property;

  /**
   * RContact:first-name:
   *
   *  contact's fist name
   */
  pspec = g_param_spec_string("first-name", 
			      "first name",
			      "contact's given name",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_FIRST, pspec);

  /**
   * RContact:middle-name:
   *
   *  contact's middle name
   */
  pspec = g_param_spec_string("middle-name", 
			      "middle name",
			      "contact's middle name",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_MIDDLE, pspec);

  /**
   * RContact:last-name:
   *
   *  contact's last name
   */
  pspec = g_param_spec_string("last-name", 
			      "last name",
			      "contact's last name",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_LAST, pspec);

  /**
   * RContact:nick-name:
   *
   *  contact's nick name
   */
  pspec = g_param_spec_string("nick-name", 
			      "nick name",
			      "contact's nick name",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_NICK, pspec);

  /**
   * RContact:profession:
   *
   * contact's profession
   */
  pspec = g_param_spec_string("profession", 
			      "profession",
			      "contact's profession",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_PROFESSION, pspec);


  /**
   * RContact:prefix:
   *
   * contact's prefix
   */
  pspec = g_param_spec_string("prefix", 
			      "prefix",
			      "contact's prefix",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_PREFIX, pspec);

  /**
   * RContact:title:
   *
   * contact's title
   */
  pspec = g_param_spec_string("title", 
			      "title",
			      "contact's title",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_TITLE, pspec);

  /**
   * RContact:photo:
   *
   * contact's photo
   */
  pspec = g_param_spec_string("photo", 
			      "photo",
			      "contact's photo",
			      NULL, 
			      G_PARAM_READWRITE);
  g_object_class_install_property (class, CONTACT_PHOTO, pspec);  
}


static void
r_contact_init(RContact* self)
{
  self->private = g_new(RContactPrivate, 1);
  self->private->dispose_has_run = FALSE;

  self->private->first    = NULL;
  self->private->middle   = NULL;
  self->private->last     = NULL;
  self->private->nick     = NULL;
  self->private->prof     = NULL;
  self->private->prefix   = NULL;
  self->private->title    = NULL;
  self->private->photo    = NULL;
  self->private->birthday = r_date_new();
}



static void 
r_contact_dispose (RContact* self)
{
  g_return_if_fail(IS_R_CONTACT(self));
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
r_contact_finalize (RContact* self)
{
  g_return_if_fail(IS_R_CONTACT(self));

  r_utils_free_string(self->private->first); 
  r_utils_free_string(self->private->middle);
  r_utils_free_string(self->private->last);
  r_utils_free_string(self->private->nick);
  r_utils_free_string(self->private->prof);
  r_utils_free_string(self->private->prefix);
  r_utils_free_string(self->private->title);
  r_utils_free_string(self->private->photo);

  r_date_free(self->private->birthday);

  g_free(self->private);
  self->private = NULL;
}


static void 
r_contact_set_property (GObject* obj, guint property_id,
			const GValue* value, GParamSpec* spec)
{
  RContact* self = (RContact*) obj;
  
  switch (property_id)
    {
   case CONTACT_FIRST:
     g_free(self->private->first);
     self->private->first = g_value_dup_string(value);
     break;
     
    case CONTACT_MIDDLE:
      g_free(self->private->middle);
      self->private->middle = g_value_dup_string(value);
      break; 
    
    case CONTACT_LAST:
      g_free(self->private->last);
      self->private->last = g_value_dup_string(value);
      break;

    case CONTACT_NICK:
      g_free(self->private->nick);
      self->private->nick = g_value_dup_string(value);
      break; 

    case CONTACT_PROFESSION:
      g_free(self->private->prof);
      self->private->prof = g_value_dup_string(value);     
      break; 

    case CONTACT_PREFIX:
      g_free(self->private->prefix);
      self->private->prefix = g_value_dup_string(value);
     break;

    case CONTACT_TITLE:
      g_free(self->private->title);
      self->private->title = g_value_dup_string(value);
     break;   

    case CONTACT_PHOTO:
      g_free(self->private->photo);
      self->private->photo = g_value_dup_string(value);
      break;    
      
    default:
      break;
    }
}



static void 
r_contact_get_property (GObject* obj, guint property_id,
			GValue* value, GParamSpec* spec)
{
  RContact* self = (RContact*) obj;
  
  switch (property_id)
    {
   case CONTACT_FIRST:
     g_value_set_string(value, self->private->first);
      break;
    
    case CONTACT_MIDDLE:
      g_value_set_string(value, self->private->middle);
      break; 
    
    case CONTACT_LAST:
      g_value_set_string(value, self->private->last);
      break;

    case CONTACT_NICK:
      g_value_set_string(value, self->private->nick);
    break; 

    case CONTACT_PROFESSION:
      g_value_set_string(value, self->private->prof);
     break; 

    case CONTACT_PREFIX:
      g_value_set_string(value, self->private->prefix);
      break;

    case CONTACT_TITLE:
      g_value_set_string(value, self->private->title);
      break;   

    case CONTACT_PHOTO:
      g_value_set_string(value, self->private->photo);
      break;    
       
    default:
      break;
    }
}


/**
 * r_contact_new
 *
 * create a new #RContact
 *
 * Returns: a #RContact*
 */
RContact*
r_contact_new(void)
{
  RContact* contact;

  contact = g_object_new(r_contact_get_type(), NULL);

  return contact;
}


/**
 * r_contact_free
 * @contact: a #RContact
 *
 * free memory owned by #RContact
 */
void
r_contact_free(RContact* contact)
{
  g_return_if_fail(IS_R_CONTACT(contact));

  g_object_unref(contact);   
}



/**
 * r_contact_set_birthday
 * @contact: a #RContact
 * @day: contact's birth day
 * @month: contact's birth month
 * @year: contact's birth year
 *
 * set contact's birthday
 */
void 
r_contact_set_birthday (RContact* contact, 
			gint day, gint month, gint year)
{
  g_return_if_fail(IS_R_CONTACT(contact));

  r_date_set_date(contact->private->birthday, day, month, year);
}


/**
 * r_contact_get_birth_day
 * @contact: a #RContact
 *
 * get contact's day of birth. Caller must free the returned value
 *
 * returns: a gchar*
*/
gchar* 
r_contact_get_birth_day (RContact* contact)
{
  gchar* ret;
  g_return_val_if_fail(IS_R_CONTACT(contact), NULL);
  
  g_object_get(contact->private->birthday, "day", &ret, NULL);

  return ret;
}
 

/**
 * r_contact_get_bith_month
 * @contact: a #RContact
 *
 * get contact's month of birth. Caller must free the returned value
 *
 * returns: a gchar*
 */ 
gchar* 
r_contact_get_birth_month (RContact* contact)
{
  gchar* ret;
  g_return_val_if_fail(IS_R_CONTACT(contact), NULL);

  g_object_get(contact->private->birthday, "month", &ret, NULL);

  return ret;
}


/**
 * r_contact_get_birth_year
 * @contact: a #RContact
 *
 * get contact's year of birth. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar* 
r_contact_get_birth_year (RContact* contact)
{
  gchar* ret;

  g_return_val_if_fail(IS_R_CONTACT(contact), NULL);
  
  g_object_get(contact->private->birthday, "year", &ret, NULL);
 
  return ret;
}


/**
 * r_contact_get_birthdate
 * @contact: a #RContact
 *
 * get contact's birthday, or "unknown" if birthday 
 * wasn't setted. Caller must free the returned value
 *
 * returns: a gchar*
 */
gchar* 
r_contact_get_birthdate (RContact* contact)
{
  g_return_val_if_fail(IS_R_CONTACT(contact), "unknown");

  return r_date_get_human_date(contact->private->birthday); 
}


/**
 * r_contact_copy
 * @contact: a #RContact
 *
 * make a copy of given object
 *
 * returns: a new #RContact object
 */
RContact* 
r_contact_copy (RContact* contact)
{
  RContact* new;
  gchar *first, *middle, *last, *nick;
  gchar *prof, *prefix, *title, *photo;
  
  g_return_val_if_fail(IS_R_CONTACT(contact), NULL);

  new = r_contact_new();
  
  g_object_get(G_OBJECT(contact), 
	       "first-name", &first, "middle-name", &middle, 
	       "last-name", &last, "nick-name", &nick, "profession", &prof, 
	       "prefix", &prefix, "title", &title, "photo", &photo, NULL);
  
  g_object_set(G_OBJECT(new), 
	       "first-name", first, "middle-name", middle, "last-name", last, 
	       "nick-name", nick, "profession",  prof, "prefix", prefix, 
	       "title", title, "photo", photo, NULL);  

  return new;
}
