/*
*  RAL -- Rubrica Addressbook Library
*  file: date.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <stdlib.h>
#include <glib.h>
#include <glib-object.h>

#include "date.h"
#include "utils.h"

#define BUFLEN 64

/*   property enumeration
*/
enum {
  DATE_KNOW = 1,
  DATE_DAY,
  DATE_MONTH,
  DATE_YEAR,
  DATE_HUMAN
};

struct _RDatePrivate {
  gboolean know;      /* is date khnown? */

  gchar* day;                
  gchar* month;
  gchar* year; 

  gchar* human_date;
  
  gboolean dispose_has_run;
};



static void r_date_class_init (RDateClass* klass);
static void r_date_init       (RDate* obj);

static void r_date_dispose    (RDate* obj);
static void r_date_finalize   (RDate* obj);


static void r_date_set_property (GObject* obj, guint property_id,
				       const GValue* value, GParamSpec* spec);

static void r_date_get_property (GObject* obj, guint property_id,
				 GValue* value, GParamSpec* spec);


GType
r_date_get_type()
{
  static GType r_date_type = 0;
  
  if (!r_date_type)
    {
      static const GTypeInfo r_date_info =
	{
	  sizeof(RDateClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) r_date_class_init,
	  NULL,
	  NULL,
	  sizeof(RDate),
	  0,
	  (GInstanceInitFunc) r_date_init
	};

      r_date_type = g_type_register_static (G_TYPE_OBJECT, "RDate",
						  &r_date_info, 0);
    }
  
  return r_date_type;
}


static void
r_date_class_init(RDateClass* klass)
{
  GObjectClass *class;
  GParamSpec* pspec;
  
  class  = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) r_date_dispose;
  class->finalize = (GObjectFinalizeFunc) r_date_finalize;
  
  class->set_property = r_date_set_property;
  class->get_property = r_date_get_property;

  /**
   * RDate:know:
   *
   * 
   */
  pspec = g_param_spec_boolean("know", 
			       "know", 
			       "is date known", 
			       FALSE,
			       G_PARAM_READWRITE);
  g_object_class_install_property (class, DATE_KNOW, pspec);

  /**
   * RDate:day:
   *
   * day
   */
  pspec = g_param_spec_string("day", 
			      "day", 
			      "the day", 
			      "BadDay", 
			      G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_DAY, pspec);

  /**
   * RDate:month:
   *
   * month
   */
  pspec = g_param_spec_string("month", 
			      "month", 
			      "the month", 
			      "BadMonth", 
			      G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_MONTH, pspec);

  /**
   * RDate:year:
   *
   * year
   */
  pspec = g_param_spec_string("year", 
			      "year", 
			      "the year", 
			      "BadYear", 
			      G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_YEAR, pspec);

  /**
   * RDate:human-date:
   *
   * date in a human readable format
   */
  pspec = g_param_spec_string("human-date",
			      "human readable",
			      "data displayed in a human readable format",
			      NULL, 
			      G_PARAM_READABLE);
  g_object_class_install_property (class, DATE_HUMAN, pspec);
}


static void
r_date_init(RDate* self)
{
  self->private = g_new(RDatePrivate, 1);

  self->private->know   = FALSE;
  
  self->private->day    = g_strdup("BadDay");   
  self->private->month  = g_strdup("BadMonth");
  self->private->year   = g_strdup("BadYear");

  self->private->human_date  = NULL;
  
  self->private->dispose_has_run = FALSE;
}


static void 
r_date_dispose (RDate* self)
{
  g_return_if_fail(IS_R_DATE(self));
  
#ifdef DEBUG
  g_print("\n\t\tr_date_dispose");
#endif
  
  if (self->private->dispose_has_run)
    return;

  self->private->dispose_has_run = TRUE;
}


static void 
r_date_finalize (RDate* self)
{
  g_return_if_fail(IS_R_DATE(self));

#ifdef DEBUG
  g_print("\n\t\tr_date_finalize");
#endif

  r_utils_free_string(self->private->day);
  r_utils_free_string(self->private->month);
  r_utils_free_string(self->private->year);
  r_utils_free_string(self->private->human_date);

  g_free(self->private);
  self->private = NULL;
}


static void 
r_date_set_property (GObject* obj, guint property_id,
		     const GValue* value,  GParamSpec* spec)
{
  /* do nothing */ 
  return;
}


static void 
r_date_get_property (GObject* obj, guint property_id,
		     GValue* value, GParamSpec* spec)
{
  RDate *self = (RDate *) obj;

  switch(property_id)
    {
    case DATE_KNOW:
      g_value_set_boolean(value, self->private->know);
      break;
      
    case DATE_DAY:
      g_value_set_string(value, self->private->day);
      break;
      
    case DATE_MONTH:
      g_value_set_string(value, self->private->month);
      break;
      
    case DATE_YEAR:
      g_value_set_string(value, self->private->year);
      break;
      
    case DATE_HUMAN:
      g_value_set_string(value, self->private->human_date);
      break;
      
    default:
      break;
    }
}



/**
 * r_date_new
 *
 * create a new #RDate object
 *
 * Returns: a #RDate
 **/
RDate*
r_date_new(void)
{
  RDate* date;

  date = g_object_new(r_date_get_type(), NULL);

  return date;
}


/**
 * r_date_free
 * @date: a #RDate
 *
 * free the RDate object
 */
void
r_date_free(RDate* date)
{
  g_return_if_fail(IS_R_DATE(date));

  g_object_unref(date);   
}


/**
 * r_date_know_date
 * @date: a #RDate
 * @know: gboolean
 *
 * Set the know date flag. Set to %TRUE if the date is known, %FALSE otherwise
 */
void 
r_date_know_date (RDate* date, gboolean know)
{
  g_return_if_fail(IS_R_DATE(date));

  date->private->know = know;
}


/**
 * r_date_set_date
 * @date: a #RDate
 * @day:
 * @month:
 * @year:
 *
 * set the data values.
 */
void 
r_date_set_date (RDate* date, gint day, gint month, gint year)
{
  struct tm *tm;
  time_t t;
  gchar buffer[BUFLEN];

  g_return_if_fail(IS_R_DATE(date));
  
  date->private->know = FALSE;
  r_utils_free_string(date->private->day);
  r_utils_free_string(date->private->month);
  r_utils_free_string(date->private->year);

  date->private->day   = g_strdup("BadDay");   
  date->private->month = g_strdup("BadMonth");
  date->private->year  = g_strdup("BadYear");
  
  if ((day < 0) || (day > 31))
    return;
  
  /* 0 < tm_mon < 11 ==> 0 is january, 11 is december */
  if ((month < 0) || (month > 12)) 
    return;
    
  date->private->know = TRUE;

  t = time(NULL);
  tm = localtime(&t);
  tm->tm_mday = day;
  tm->tm_mon  = month - 1;
  tm->tm_year = year;

  r_utils_free_string(date->private->day);
  r_utils_free_string(date->private->month);
  r_utils_free_string(date->private->year);
  
  date->private->day   = g_strdup_printf("%d", tm->tm_mday);
  date->private->month = g_strdup_printf("%d", tm->tm_mon + 1);
  date->private->year  = g_strdup_printf("%d", tm->tm_year);  
  
  strftime(buffer, BUFLEN, "%Ex", tm);
  
  r_utils_free_string(date->private->human_date);
  date->private->human_date = g_strdup(buffer);
}
 


/**
 * r_date_get_human_date
 * @date: a #RDate
 *
 * get the date as a locale string. Caller must free the returned value
 *
 * returns: a gchar* or "unknown", if date is not set
 */
gchar*       
r_date_get_human_date (RDate* date)
{
  g_return_val_if_fail(IS_R_DATE(date), g_strdup(_("unknown")));

  return g_strdup(date->private->human_date);
}


/**
 * r_date_is_date_known
 * @date: a #RDate
 *
 * get the state of know date flag
 *
 * returns: a boolean
 */ 
gboolean 
r_date_is_date_known (RDate* date)
{
  g_return_val_if_fail(IS_R_DATE(date), FALSE);

  return date->private->know;
}



/**
 * r_date_copy
 * @date: a #RDate
 * 
 * copy the given #RDate
 * 
 * returns: a new allocated RDate*
 */
RDate* 
r_date_copy (RDate* date)
{
  RDate* new;
  gchar *day, *month, *year;

  g_return_val_if_fail(IS_R_DATE(date), NULL);

  new = r_date_new();
  
  g_object_get(date, "day", &day, "month", &month, "year", &year, NULL);

  r_date_know_date(new, date->private->know);
  r_date_set_date(new, atoi(day), atoi(month), atoi(year));

  return new;
}
