/*
*  RAL -- Rubrica Addressbook Library
*  file: groups.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <glib.h>
#include <glib-object.h>

#include "group_set.h"
#include "group.h"

enum {
  GROUP_ADDED,
  GROUP_REMOVED,

  LAST_SIGNAL 
};

static guint r_group_set_signals[LAST_SIGNAL] = {0};


struct _RGroupSetPrivate {
  GList* lst;
  GList* iter;
  GList* find;

  gboolean dispose_has_run;
};


static void r_group_set_class_init (RGroupSetClass* klass);
static void r_group_set_init       (RGroupSet* obj);

static void r_group_set_dispose    (RGroupSet* obj);
static void r_group_set_finalize   (RGroupSet* obj);



GType
r_group_set_get_type()
{
  static GType group_set_type = 0;
  
  if (!group_set_type)
    {
      static const GTypeInfo group_set_info =
	{
	  sizeof(RGroupSetClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) r_group_set_class_init,
	  NULL,
	  NULL,
	  sizeof(RGroupSet),
	  0,
	  (GInstanceInitFunc) r_group_set_init
	};

      group_set_type = g_type_register_static (G_TYPE_OBJECT, 
					       "RGroupSet",
					       &group_set_info, 0);
    }
  
  return group_set_type;
}


static void
r_group_set_class_init(RGroupSetClass* klass)
{
  GObjectClass *class; 

  class           = G_OBJECT_CLASS (klass);
  class->dispose  = (GObjectFinalizeFunc) r_group_set_dispose;
  class->finalize = (GObjectFinalizeFunc) r_group_set_finalize;


  /* class signals */
  /**
   * RGroupSet::group_added
   * @set: the #RGroupSet that receives the signal
   * @group: the group added to the set
   *
   * signal emitted when a new group is added to the group's set.
   */
  r_group_set_signals[GROUP_ADDED] =
    g_signal_new("group_added", 
		 R_GROUP_SET_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RGroupSetClass, group_added),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_POINTER);        /* params type */

  /**
   * RGroupSet::group_removed
   * @set: the #RGroupSet that receives the signal
   * @group: the group removed from the set
   *
   * signal emitted when a group is removed from the group's set.
   */
  r_group_set_signals[GROUP_REMOVED] =
    g_signal_new("group_removed", 
		 R_GROUP_SET_TYPE,
		 G_SIGNAL_RUN_LAST,
		 G_STRUCT_OFFSET(RGroupSetClass, group_removed),
		 NULL,
		 NULL,
		 g_cclosure_marshal_VOID__INT,
		 G_TYPE_NONE,            /* return type */
		 1,                      /* params      */
		 G_TYPE_POINTER);        /* params type */
}


static void
r_group_set_init(RGroupSet* self)
{
  self->private = g_new(RGroupSetPrivate, 1);

  self->private->lst  = NULL;
  self->private->iter = NULL;
  self->private->dispose_has_run = FALSE;
}



static void 
r_group_set_dispose (RGroupSet* self)
{
  GList* list;

  g_return_if_fail(IS_R_GROUP_SET(self));
  
  if (self->private->dispose_has_run)
    return;

  for (list = self->private->lst; list; list = g_list_next(list))
    r_group_free(R_GROUP(list->data));   
  
  g_list_free(self->private->lst);

  self->private->dispose_has_run = TRUE;
}


static void 
r_group_set_finalize (RGroupSet* self)
{
  g_return_if_fail(IS_R_GROUP_SET(self));
  
  g_free(self->private);
  self->private = NULL;
}



/*    Public
*/

/**
 * r_group_set_
 * @set: a #RGroupSet
 *
 * create a new #RGroupSet
 *
 * returns: a new allocated #RGroupSet*
 */
RGroupSet*
r_group_set_new(void)
{
  RGroupSet* set;

  set = g_object_new(r_group_set_get_type(), NULL);

  return set;
}


/**
 * r_group_set_free
 * @set: a #RGroupSet
 *
 * free memory owned by a #RGroupSet
 */
void
r_group_set_free(RGroupSet* set)
{
  g_return_if_fail(IS_R_GROUP_SET(set));

  g_object_unref(set);   
}


/**
 * r_group_set_is_empty
 * @set: a #RGroupSet
 * 
 * Test if the given group_set is empty
 *
 * Returns: %TRUE if the set is empty, %FALSE otherwise
 */
gboolean 
r_group_set_is_empty (RGroupSet* set)
{
   g_return_val_if_fail(IS_R_GROUP_SET(set), TRUE);  
   
   return (set->private->lst == NULL);
}


/**
 * r_group_set_find
 * @set: a #RGroupSet
 * @group_name: a #RGroup's name
 *
 * find a group by name
 * 
 * returns: a gpoiter, to the group with the given name or NULL, if 
 * set haven't the group. 
 */
gpointer  
r_group_set_find (RGroupSet* set, gchar* group_name) 
{ 
  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL);  
  g_return_val_if_fail(group_name != NULL, NULL); 

  set->private->find = set->private->lst;
  for (; set->private->find; set->private->find = set->private->find->next) 
    {
      RGroup* grp = set->private->find->data;

      if (r_group_has_name(R_GROUP(grp), group_name)) 
	return (gpointer) grp;
    } 
  
  return NULL;
}


/**
 * r_group_set_find_groups_owned_by
 * @set: a #RGroupSet
 * @owner: a gchar*
 *
 * search for all groups owned by the given "owner"
 *
 * Returns: a #GList of #RGroup or NULL
 */
GList* 
r_group_set_find_groups_owned_by (RGroupSet* set, const gchar* owner)
{
  GList* ret = NULL;
 
  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL);  
  g_return_val_if_fail(owner != NULL, NULL); 
  
  set->private->find = set->private->lst;
  for (; set->private->find; set->private->find = set->private->find->next) 
    {
      RGroup* grp = set->private->find->data;

      if (r_group_has_owner(R_GROUP(grp), owner)) 
	ret = g_list_append(ret, grp);
    } 
  
  return ret;  
}


/**
 * r_group_set_owns_group
 * @set: a #RGroupSet
 * @group_name: a #RGroup's name
 *
 * test if set owns the given group
 *
 * returns: a gboolean, %TRUE if set owns group, %FALSE otherwise
 */
gboolean 
r_group_set_owns_group (RGroupSet* set, gchar* group_name)
{
  g_return_val_if_fail(IS_R_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(group_name != NULL, FALSE); 

  if (r_group_set_find(set, group_name)) 
    return TRUE; 
  
  return FALSE;   
}


/**
 * r_group_set_add_group
 * @set: a #RGroupSet
 * @group: a #RGroup
 *
 * add the group to te set
 *
 * returns: a gboolean. %TRUE if group if added successfully, %FALSE otherwise
 */
gboolean               
r_group_set_add_group (RGroupSet* set, RGroup* group) 
{
  gchar* name; 

  g_return_val_if_fail(IS_R_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(IS_R_GROUP(group), FALSE); 

  /* test if group is already known 
   */
  g_object_get(group, "group-name", &name, NULL);
  if (r_group_set_owns_group(set, name))
    return TRUE;
  
  /* else add to the set
   */
  set->private->lst = g_list_append(set->private->lst, group);

  /* init group iterator */
  if (!set->private->iter)
    set->private->iter = set->private->lst;  

  g_signal_emit_by_name(set, "group_added", group, G_TYPE_POINTER);

  return TRUE;
}


/**
 * r_group_set_delete_group_by_name
 * @set: a #RGroupSet
 * @name: group's name
 *
 * delete a group from set
 *
 * returns: a gboolean. %TRUE if group is deleted, %FALSE otherwise
 */
gboolean     
r_group_set_delete_group_by_name (RGroupSet* set, gchar* name)
{
  g_return_val_if_fail(IS_R_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(name != NULL, FALSE);
 
  set->private->iter = set->private->lst;
  for (; set->private->iter; set->private->iter = set->private->iter->next) 
    { 
      RGroup* group = set->private->iter->data;
      
      if (r_group_has_name(R_GROUP(group), name)) 
	{ 
	  set->private->lst = g_list_remove_link(set->private->lst,
						  set->private->iter);
	  
	  r_group_free(R_GROUP(set->private->iter->data));
	  
	  g_list_free_1(set->private->iter);
	  set->private->iter = NULL;	  

	  return TRUE;
	}
    }
  
  return FALSE;
}


/**
 * r_group_set_delete_group
 * @set: a #RGroupSet
 * @group: a #RGroup
 *
 * delete a group from set
 *
 * returns: a gboolean. %TRUE if group is deleted, %FALSE otherwise
 */
gboolean     
r_group_set_delete_group (RGroupSet* set, RGroup* group)
{
  gchar* name;

  g_return_val_if_fail(IS_R_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(IS_R_GROUP(group), FALSE);

  g_object_get(group, "group-name", &name, NULL);
 
  return r_group_set_delete_group_by_name(set, name);
}


/**
 * r_group_set_rename_group
 * @set: a #RGroupSet
 * @oldname:
 * @newname:
 *
 * rename a group in the set
 *
 * returns: a gboolean. %TRUE if group is renamed, %FALSE otherwise
 */
gboolean 
r_group_set_rename_group (RGroupSet* set, gchar* oldname, gchar* newname)
{
  RGroup* group;

  g_return_val_if_fail(IS_R_GROUP_SET(set), FALSE); 
  g_return_val_if_fail(oldname != NULL, FALSE);
  g_return_val_if_fail(newname != NULL, FALSE);

  group = r_group_set_find(set, oldname);
  if (r_group_rename(group, newname))
    return TRUE;
  
  return FALSE;
}



/**
 * r_group_set_disable_all
 * @set: a #RGroupSet
 *
 * disable all groups in set 
 */
void     
r_group_set_disable_all (RGroupSet* set)
{
  RGroup* group;

  g_return_if_fail(IS_R_GROUP_SET(set)); 
  
  r_group_set_reset(set);
  group = r_group_set_get_group(set);
  for(; group; group = r_group_set_get_next_group(set))
    g_object_set(group, "enabled", FALSE, NULL);
}


/**
 * r_group_set_enable_group
 * @set: a #RGroupSet
 * @group: a #RGroup
 * 
 * enable the given group
 */
void     
r_group_set_enable_group (RGroupSet* set, RGroup* grp)
{
  gchar* name;
  RGroup* group = NULL;

  g_return_if_fail(IS_R_GROUP_SET(set)); 
  g_return_if_fail(IS_R_GROUP(grp));

  g_object_get(grp, "group-name", &name, NULL);
  group = r_group_set_find(set, name);

  if (group)
    g_object_set(group, "enabled", TRUE, NULL);
}


/**
 * r_group_set_disable_group
 * @set: a #RGroupSet
 * @group: a #RGroup
 * 
 * disable the given group
 */
void     
r_group_set_disable_group (RGroupSet* set, RGroup* grp)
{
  gchar* name;
  RGroup* group = NULL;

  g_return_if_fail(IS_R_GROUP_SET(set)); 
  g_return_if_fail(IS_R_GROUP(grp));
  
  g_object_get(grp, "group-name", &name, NULL);
  group = r_group_set_find(set, name);
  if (group)
    g_object_set(group, "enabled", FALSE, NULL); 
}




/**
 * r_group_set_merge_sets
 * @set: a #RGroupSet
 * @second: a #RGroupSet
 *
 * merges the #RGroupSets
 *
 *returns: a RGroupSet*
 */
RGroupSet* 
r_group_set_merge_sets (RGroupSet* set, RGroupSet* second)
{
  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL); 
  g_return_val_if_fail(IS_R_GROUP_SET(second), set);
 
  set->private->lst = g_list_concat(set->private->lst, second->private->lst);
  
  return set;
}


/**
 * r_group_set_reset
 * @set: a #RGroupSet
 *
 * reset the private #RGroupSet iterator to the head of group's list.
 */
void
r_group_set_reset(RGroupSet* set)
{
  g_return_if_fail(IS_R_GROUP_SET(set));
  
  set->private->iter = set->private->lst;
}


/**
 * r_group_set_get_group
 * @set: a #RGroupSet
 *
 * get the first group in set
 *
 * returns: a gpointer or NULL if set is void
 */
RGroup* 
r_group_set_get_group (RGroupSet* set)
{
  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL); 
  
  if (set->private->iter)
    return (RGroup*) set->private->iter->data;
  
  return NULL;
}


/**
 * r_group_set_get_next_group
 * @set: a #RGroupSet
 *
 * get the next group in set. 
 *
 * returns: a gpointer or NULL if the groups's end list is reached.
 */
RGroup* 
r_group_set_get_next_group (RGroupSet* set)
{
  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL); 

  set->private->iter = g_list_next(set->private->iter);
  if (set->private->iter)
    return (RGroup*) set->private->iter->data;
  else
    set->private->iter = set->private->lst;

  return NULL;
}


/**
 * r_group_set_get_prev_group
 * @set: a #RGroupSet
 *
 * get the previous group in set.
 *
 * returns: a gpointer or NULL if the groups's head list is reached.
 */
RGroup* 
r_group_set_get_prev_group (RGroupSet* set)
{
  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL); 

  g_return_val_if_fail(IS_R_GROUP_SET(set), NULL); 

  set->private->iter = g_list_previous(set->private->iter);
  if (set->private->iter)
    return (RGroup*) set->private->iter->data;
  else
    set->private->iter = set->private->lst;

  return NULL;
}







