/*
*  file: manager.c
*
*  
*  Copyright (C) 2005 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <glib.h>
#include <glib-object.h>
#include <string.h>
#include <libintl.h>

#include "manager.h"
#include "plugin.h"
#include "types.h"

#define _(x) gettext(x)

struct _RPluginManagerPrivate {
  GList* plugins;

  gboolean dispose_has_run;
};



static void r_plugin_manager_class_init (RPluginManagerClass* klass);
static void r_plugin_manager_init       (RPluginManager* obj);

static void r_plugin_manager_dispose    (RPluginManager* obj);
static void r_plugin_manager_finalize   (RPluginManager* obj);



GType
r_plugin_manager_get_type()
{
  static GType manager_type = 0;
  
  if (!manager_type)
    {
      static const GTypeInfo manager_info =
	{
	  sizeof(RPluginManagerClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) r_plugin_manager_class_init,
	  NULL,
	  NULL,
	  sizeof(RPluginManager),
	  0,
	  (GInstanceInitFunc) r_plugin_manager_init
	};
      
      manager_type = g_type_register_static (G_TYPE_OBJECT, "PluginManager",
					     &manager_info, 0);
    }
  
  return manager_type;
}


static void
r_plugin_manager_class_init(RPluginManagerClass* klass)
{
  GObjectClass *class;
  
  class = G_OBJECT_CLASS (klass);

  class->dispose  = (GObjectFinalizeFunc) r_plugin_manager_dispose;
  class->finalize = (GObjectFinalizeFunc) r_plugin_manager_finalize;
}


static void
r_plugin_manager_init(RPluginManager* manager)
{  
  manager->private = g_new(RPluginManagerPrivate, 1);
  
  manager->private->plugins = NULL;
  manager->private->dispose_has_run = FALSE;
}


static void 
r_plugin_manager_dispose (RPluginManager* manager)
{
  g_return_if_fail(R_IS_PLUGIN_MANAGER(manager));
  
  if (manager->private->dispose_has_run)
    return;

  manager->private->dispose_has_run = TRUE;
}


static void 
r_plugin_manager_finalize (RPluginManager* manager)
{
  g_return_if_fail(R_IS_PLUGIN_MANAGER(manager));
  
  g_free(manager->private);
  manager->private = NULL;
}


/*    Public
*/

/**
 * r_plugin_manager_new
 *
 * create a new plugin manager
 *
 * Returns: a #RPluginManager*
 */
RPluginManager* 
r_plugin_manager_new(void)
{
  RPluginManager* manager;

  manager = g_object_new(r_plugin_manager_get_type(), NULL);

  return manager;
}


/**
 * r_plugin_manager_free
 * @manager: a #RPluginManager
 *
 * free the object
 */
void
r_plugin_manager_free(RPluginManager* manager)
{
  g_return_if_fail(R_IS_PLUGIN_MANAGER(manager));

  g_object_unref(manager);
}

/**
 * r_plugin_manager_scan_directory
 * @manager: a #RPluginManager
 * @dirname: a gchar*
 *
 * scan the given directory lookinf for RAL's plugins
 */
void 
r_plugin_manager_scan_directory (RPluginManager* manager, gchar* dirname)
{
  GDir* dir;
  GError* error;
  const gchar* file;
  RPlugin* plugin;

  g_return_if_fail(R_IS_PLUGIN_MANAGER(manager));
  
  dir = g_dir_open(dirname, 0, &error);
  if (error)
    {
      // TODO
    }

  while ((file = g_dir_read_name(dir)) != NULL)
    {
      if (g_str_has_suffix(file, ".so"))
	{
	  plugin = r_plugin_new();
	  
	  if (!r_plugin_load(plugin, file))
	    {
	      g_warning(_("\nerror loading plugin %s"), file);
	    }
	  else
	    manager->private->plugins = 
	      g_list_append(manager->private->plugins, plugin);
	}
    }
}


/**
 * r_plugin_manager_how_many
 * @manager: a #RPluginManager
 *
 * returns the number of plugins that have been read
 * 
 * Returns: a gint
 */
gint 
r_plugin_manager_how_many (RPluginManager* manager)
{
  g_return_val_if_fail(R_IS_PLUGIN_MANAGER(manager), 0);
 
  return g_list_length(manager->private->plugins);
}


/**
 * r_plugin_manager_get_nth_plugin
 * @manager: a #RPluginManager
 * @n: a gint
 *
 * get the n-th plugin
 *
 * Returns: a #RPlugin* or %NULL
 */
RPlugin* 
r_plugin_manager_get_nth_plugin (RPluginManager* manager, gint n)
{
  g_return_val_if_fail(R_IS_PLUGIN_MANAGER(manager), NULL);
  g_return_val_if_fail(n >= 0, NULL);
  g_return_val_if_fail(n < g_list_length(manager->private->plugins), NULL);
  
  return g_list_nth_data(manager->private->plugins, n);
}


/**
 * r_plugin_manager_get_plugin
 * @manager: a #RPluginManager
 * @name: a gchar*
 *
 * get the plugin with the given name
 *
 * Returns: a #RPlugin* or %NULL if plugin is not found
 */
RPlugin* 
r_plugin_manager_get_plugin (RPluginManager* manager, const gchar* name)
{
  GList *iter;

  g_return_val_if_fail(R_IS_PLUGIN_MANAGER(manager), NULL);
  g_return_val_if_fail(name != NULL, NULL);
  
  iter = manager->private->plugins;
  for (; iter; iter = g_list_next(iter))
    {
      RPlugin* plugin = (RPlugin*) iter->data;;
      gchar* plugin_name;
      
      if (plugin)
	{
	  g_object_get(plugin, "plugin-name", &plugin_name, NULL);
	  
	  if (g_ascii_strcasecmp(plugin_name, name) == 0)
	    return plugin;
	}
    }

  return NULL;
}


/**
 * r_plugin_manager_get_all_filters
 * @manager: a #RPluginManager
 * 
 * get all filters mamaged by installed plugin
 *
 * Returns: a #GList of #RFilters or NULL
 */
GList* 
r_plugin_manager_get_all_filters (RPluginManager* manager)
{
  gint n, i;
  RPlugin* plugin;
  GList* filters = NULL;

  g_return_val_if_fail(R_IS_PLUGIN_MANAGER(manager), NULL);
  
  n = g_list_length(manager->private->plugins);
  for (i = 0; i < n; i++)
    {
      GList* plugin_filters, *l;
      RFilter* filter;
      
      plugin  = r_plugin_manager_get_nth_plugin(manager, i);
      plugin_filters = r_plugin_get_filters(plugin);
      
      for (l = plugin_filters; l; l = l->next)
	{
	  filter  = r_filter_copy(R_FILTER(l->data));
	  filters = g_list_append(filters, filter);
	}
    }
  
  return filters;
}
